//*****************************************************************************
// File Name	: iPodController.c
// 
// Title		: iPodController
// Revision		: 1.0
// Notes		:	
// Target MCU	: Atmel AVR AT90S2313
// Editor Tabs	: 4
// 
// Revision History:
// When			Who			Description of change
// -----------	-----------	-----------------------
// 20-Feb-2007	Junichi Nagayama	Created the program
//*****************************************************************************


//	Port pins
//	1:xReset		Reset
//	2:PD0(RxD)		to iPod TxD
//	3:PD1(TxD)		to iPod RxD
//	4:XTAL2			Xtal 3.6864
//	5:XTAL1			Xtal 3.6864
//	6:PD2(INT0)		xPlay PI
//	7:PD3(INT1)		xInhibit PI
//	8:PD4(T0)		
//	9:PD5(T1)		
//	10:GND
//	11:PD6(ICP)		
//	12:PB0(AIN0)	LED1
//	13:PB1(AIN1)	LED2
//	14:PB2
//	15:PB3(OC1)
//	16:PB4
//	17:PB5(MOSI)
//	18:PB6(MISO)
//	19:PB7(SCK)
//	20:VCC


//----- Include Files ---------------------------------------------------------
#include <avr/io.h>		// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support

#include "global.h"		// include our global settings

void iPodController(void);

volatile char int0, timer0, uartSent;
volatile char command[27];
volatile int dataCnt, commandLen;

#define INHIBIT	((PIND & 0x08) == 0)
#define PLAY	((PIND & 0x04) == 0)

#define PLAYING 0
#define PAUSING 1

#define LED1_ON		cbi(PORTB, 1)
#define LED1_OFF	sbi(PORTB, 1)
#define LED2_ON		cbi(PORTB, 0)
#define LED2_OFF	sbi(PORTB, 0)


//----- Begin Code ------------------------------------------------------------

SIGNAL(SIG_INTERRUPT0)
{
	int0 = 1;
}

SIGNAL(SIG_OVERFLOW0)
{
	timer0 = 1;
	TCNT0 = 112;
}

SIGNAL(SIG_UART_DATA) {//TRANS) { //SIG_UART_DATA
	if (dataCnt > 0) {
		UDR = command[commandLen - dataCnt];
		dataCnt--;
	}
}

int main(void)
{
	cli();

	// initialize
	MCUCR = 0b00001010;
		 
	// set port pins
	outb(PORTB, 0b00000000);
	outb(DDRB,  0b00000011);
	outb(PORTD, 0b00001100);
	outb(DDRD,  0b00000010);

	outb(GIMSK, 0b11000000);
	outb(TIMSK, 0b00000010); // Disable Timer0 interrupt
	TCCR0 = 0x04; // Timer0 Prescaler to /256
	TCNT0 = 112;

	GIFR = 0x40;
	UBRR = 11; // 3686400 / 16(11 + 1) = 19200bps
	UCR = 0b00011010;

	sei();

	// begin main loop
	iPodController();

	return 0;
} 

void iPodController(void)
{
	int stateGuard, lastState, cmd;

	uartSent = 1;
	dataCnt = 0;
	commandLen = 0;
	cmd = 0;
	stateGuard = 0;
	lastState = PLAYING;

	LED1_OFF;
	LED2_OFF;

	while(1) {
		if (PLAY && lastState == PAUSING) {
			if (stateGuard == 0 && dataCnt == 0) {
				cmd = 1;
				lastState = PLAYING;
				if (!INHIBIT)
					stateGuard = 50;
			}
		} else if (!PLAY && lastState == PLAYING) {
			if (stateGuard == 0 && dataCnt == 0) {
				cmd = 2;
				lastState = PAUSING;
				if (!INHIBIT)
					stateGuard = 50;
			}
		}

		if (INHIBIT) {
			cmd = 0;
			LED1_OFF;
			LED2_OFF;
		} else if (cmd == 1) {
			command[0] = 0xff;	//header0
			command[1] = 0x55;	//header1
			command[2] = 0x04;	//length
			command[3] = 0x02;	//mode
			command[4] = 0x00;	//command1
			command[5] = 0x00;	//command2
			command[6] = 0x01;	//command3
			command[7] = 0xf9;	//sum

			command[8]  = 0xff;	//header0
			command[9]  = 0x55;	//header1
			command[10] = 0x03;	//length
			command[11] = 0x02;	//mode
			command[12] = 0x00;	//command1
			command[13] = 0x00;	//command2
			command[14] = 0xfb;	//sum
			
			dataCnt = 15;
			commandLen = 15;
			
			LED1_ON;
			LED2_OFF;

			cmd = 0;

			sbi(UCR, 5);
		} else if (cmd == 2) {
			command[0] = 0xff;	//header0
			command[1] = 0x55;	//header1
			command[2] = 0x04;	//length
			command[3] = 0x02;	//mode
			command[4] = 0x00;	//command1
			command[5] = 0x00;	//command2
			command[6] = 0x02;	//command2
			command[7] = 0xf8;	//sum

			command[8]  = 0xff;	//header0
			command[9]  = 0x55;	//header1
			command[10] = 0x03;	//length
			command[11] = 0x02;	//mode
			command[12] = 0x00;	//command1
			command[13] = 0x00;	//command2
			command[14] = 0xfb;	//sum
			
			dataCnt = 15;
			commandLen = 15;
			
			LED1_OFF;
			LED2_ON;

			cmd = 0;

			sbi(UCR, 5);
		}

		if (dataCnt == 0) {
			if (stateGuard > 0)
				stateGuard--;
			timer0 = 0;
			while (!timer0) ;

			cbi(UCR, 5);
		}
	}
}
